@@ -17,7 +17,7 @@
 
 *********************************************************************************/
 
-//     zfuncs.h   version  v.6.8
+//     zfuncs.h   version  v.7.0
 
 #include <sys/sysinfo.h>
 #include <sys/time.h>
@@ -30,6 +30,7 @@
 #include <errno.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <sys/mman.h>
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -100,6 +101,9 @@ void beroot(int argc = 0, char *argv[] = 0);
 double get_seconds();                                                            //  seconds since 2000.01.01
 void start_timer(double &time0);                                                 //  start a timer
 double get_timer(double &time0);                                                 //  get elapsed time in seconds
+void ztimer_start();                                                             //  start 1 millisec. timer
+void ztimer_stop();                                                              //  stop 1 millisec. timer
+int  ztimer_milliseconds();                                                      //  get millisecs. since startup
 void start_CPUtimer(double &time0);                                              //  start a process CPU timer
 double get_CPUtimer(double &time0);                                              //  get elapsed CPU time, seconds
 double CPUtime();                                                                //  get elapsed process CPU time for main()
@@ -112,6 +116,7 @@ int  parseprocrec(char *prec, int field, double *value, ...);
 int  coretemp();                                                                 //  get CPU core temperature, deg. C
 int  disktemp(char *disk);                                                       //  get disk temp, e.g. "/dev/sda"
 void zsleep(double dsecs);                                                       //  sleep specified seconds
+void zloop(double dsecs);                                                        //  loop specified seconds
 int  global_lock(cchar *lockfile);                                               //  obtain exclusive lock, multi-process
 int  global_unlock(int fd, cchar *lockfile);                                     //  release the lock
 int  resource_lock(int &resource);                                               //  simple lock/unlock usable with GTK
@@ -120,6 +125,8 @@ int  zget_locked(int &param);
 void zput_locked(int &param, int value);                                         //  put and unlock multi-thread parameter
 int  zadd_locked(int &param, int incr);                                          //  increment multi-thread parameter
 void start_detached_thread(void * tfunc(void *), void * arg);                    //  start detached thread function
+pthread_t start_Jthread(void * threadfunc(void *), void * arg);                  //  start joinable thread function
+int  wait_Jthread(pthread_t tid);                                                //  wait for completion (join thread)
 void synch_threads(int NT = 0);                                                  //  synchronize NT threads
 int  zsystem(cchar *command);                                                    //  system() + return child process status
 int  shell_quiet(cchar *command, ...);                                           //  format/run shell command, return status
@@ -137,7 +144,9 @@ int  parsefile(cchar *path, char **dir, char **file, char **ext);
 int  renamez(cchar *file1, cchar *file2);                                        //  rename, also across file systems
 int  check_create_dir(char *path);                                               //  check if directory exists, ask to create
 int  copyFile(cchar *sfile, cchar *dfile);                                       //  copy file to file or file to directory
+int  zreaddir(cchar *directory, char **&files);                                  //  return all files in a directory, sorted
 char * combine_argvs(int argc, char *argv[], int Nth);                           //  combine argv[ii] elements from Nth to last
+char * zescape_quotes(cchar *file);                                              //  escape quote marks (") in file name
 
 //  measure CPU time spent in a function or code block within a function
 
@@ -208,6 +217,7 @@ int MatchWild(cchar * wildstr, cchar * str);
 int MatchWildIgnoreCase(cchar * wildstr, cchar * str);                           //  wildcard string match, ignoring case
 cchar * SearchWild(cchar *wpath, int &flag);                                     //  wildcard file search
 cchar * SearchWildCase(cchar *wpath, int &flag);                                 //  wildcard file search, ignoring case
+int zfind(cchar *pattern, char **&flist, int &NF);                               //  wildcard file search using glob()
 
 //  search and sort functions ===================================================
 
@@ -292,17 +302,17 @@ int  widget_font_metrics(GtkWidget *widget, int &fww, int &fhh);
 int  locale_filespec(cchar *ftype, cchar *fname, char *filespec);                //  get a locale dependent file
 void showz_userguide(cchar *context = 0);                                        //  show user guide in new process
 void showz_logfile();                                                            //  show log file in popup window
-void showz_textfile(cchar *type, cchar *file);                                   //  show text file [.gz] in popup window
+void showz_textfile(cchar *type, cchar *file, GtkWidget *parent);                //  show text file [.gz] in popup window
 void showz_html(cchar *url);                                                     //  show html via preferred browser
 
 //  translation functions =======================================================
 
-#define ZTXmaxent 2000                                                           //  max. translation strings
-#define ZTXmaxcc 4000                                                            //  max. cc per string
+#define E2Xmaxent 2000                                                           //  max. translation strings
+#define E2Xmaxcc 4000                                                            //  max. cc per string
 
-void ZTXinit(cchar *lang);                                                       //  setup for message translation
-cchar * ZTX(cchar *english);                                                     //  get translation for English message
-cchar * ZTX_missing(int &ftf);                                                   //  get missing translations, one per call
+void E2Xinit(cchar *lang);                                                       //  setup for message translation
+cchar * E2X(cchar *english);                                                     //  get translation for English message
+cchar * E2X_missing(int &ftf);                                                   //  get missing translations, one per call
 
 
 /********************************************************************************
@@ -345,6 +355,7 @@ void textwidget_clear(GtkWidget *widget);
 void textwidget_clear(GtkWidget *widget, int line);                                    //  clear text from line to end
 int  textwidget_linecount(GtkWidget *widget);                                          //  get current line count
 void textwidget_append(GtkWidget *widget, int bold, cchar *format, ...);               //  append line
+void textwidget_append2(GtkWidget *widget, int bold, cchar *format, ...);              //  append line and scroll to end
 void textwidget_insert(GtkWidget *widget, int bold, int line, cchar *format, ...);     //  insert line 
 void textwidget_replace(GtkWidget *widget, int bold, int line, cchar *format, ...);    //  replace line
 void textwidget_delete(GtkWidget *widget, int line);                                   //  delete line
@@ -448,6 +459,7 @@ void Vmenu_block(int flag);
 #define zdmaxbutts 10
 #define zdsentinel 0x97530000
 #define zdialog_max 20
+#define zdialog_button_shortcuts "Done Cancel Apply Reset"                       //  buttons that may have KB shortcuts
 
 struct zwidget  {
       cchar       *type;                           //  dialog, hbox, vbox, label, entry ...
@@ -461,8 +473,8 @@ struct zwidget  {
       int         space;                           //  extra padding space (pixels)
       int         wrap;                            //  wrap mode for edit widget
       int         rescale;                         //  widget is rescaled for more resolution
-      float       lval, nval, hval;                //  scale range and neutral value
-      float       lolim, hilim, step;              //  range and step value for "float" widget
+      double      lval, nval, hval;                //  scale range and neutral value
+      double      lolim, hilim, step;              //  range and step value for number widget
       GtkWidget   *widget;                         //  GTK widget pointer
 };
 
@@ -476,8 +488,9 @@ struct zdialog  {
       int         disabled;                        //  widget signals/events are disabled
       int         saveposn;                        //  save and recall window position each use
       int         saveinputs;                      //  save and recall user inputs each use
-      GtkWidget   *dialog;                         //  dialog window
+      GtkWidget   *dialog;                         //  dialog window or null (box parent)
       GtkWidget   *parent;                         //  parent window or null
+      cchar       *ptype;                          //  null or "window" or "box" parent
       cchar       *compbutton[zdmaxbutts];         //  dialog completion button labels
       GtkWidget   *compwidget[zdmaxbutts];         //  dialog completion button widgets
       zwidget     widget[zdmaxwidgets];            //  dialog widgets (EOF = type = 0)
@@ -513,6 +526,7 @@ int zdialog_rescale(zdialog *zd, cchar *wname, float, float, float);
 
 typedef int zdialog_event(zdialog *zd, cchar *name);                             //  widget event callback function
 int zdialog_run(zdialog *zd, zdialog_event = 0, cchar *posn = 0);                //  run dialog, handle events
+void zdialog_KB_addshortcut(cchar *key, cchar *menu);                            //  KB shortcut for zdialog compl. button
 void KBevent(GdkEventKey *event);                                                //  extern: pass KB events to main app
 
 int zdialog_send_event(zdialog *zd, cchar *event);                               //  send an event to an active dialog
@@ -559,7 +573,8 @@ void window_to_mouse(GtkWidget *window);
 GtkWidget * popup_report_open(GtkWidget *paren, cchar *titl, int ww, int hh);    //  open popup text report window
 void popup_report_header(int bold, cchar *format, ...);                          //  write non-scrolling header line
 void popup_report_write(int bold, cchar *format, ...);                           //  write text line
-void popup_report_top();                                                         //  go to top of window
+void popup_report_top();                                                         //  go to top of report window
+void popup_report_bottom();                                                      //  go to bottom of report window
 void popup_report_move(int ww, int hh);                                          //  move window ww/hh pixels
 void popup_report_close(int secs);                                               //  close window in given seconds
 
@@ -569,16 +584,18 @@ int popup_command(cchar *command, int ww = 400, int hh = 300, GtkWidget *parent
 
 //  popup message dialogs
 
-void zmessageACK(GtkWidget *parent, cchar *format, ... );                                 //  display message, wait for OK
-int  zmessageYN(GtkWidget *parent, cchar *format, ... );                                  //  display message, wait for YES/NO
-zdialog * zmessage_post(GtkWidget *parent, int secs, cchar *format, ...);                 //  show message, timeout or cancel
-zdialog * zmessage_post_bold(GtkWidget *parent, int secs, cchar *format, ...);            //   " " with big red bold font
-char * zdialog_text(GtkWidget *parent, cchar *title, cchar *inittext);                    //  get short text input from user
-int  zdialog_choose(GtkWidget *parent, cchar *message, ...);                              //  show message, return choice
-void poptext_mouse(GtkWidget *parent, cchar *text, int dx, int dy, float s1, float s2);   //  show popup text at mouse
-void poptext_window(cchar *text, GtkWindow *, int mx, int my, float s1, float s2);        //  show popup text at window posn
-int  poptext_killnow();                                                                   //  kill current popup window
-int  popup_image(cchar *imagefile, GtkWindow *parent, int Fnewin, int size);              //  popup window with image
+void zmessageACK(GtkWidget *parent, cchar *format, ... );                        //  display message, wait for OK
+int  zmessageYN(GtkWidget *parent, cchar *format, ... );                         //  display message, wait for YES/NO
+zdialog * zmessage_post(GtkWidget *, cchar *loc, int s, cchar *f, ...);          //  show message, timeout or cancel
+zdialog * zmessage_post_bold(GtkWidget *, cchar *loc, int s, cchar *f, ...);     //   " " with big red bold font
+char * zdialog_text(GtkWidget *parent, cchar *title, cchar *inittext);           //  get short text input from user
+int  zdialog_choose(GtkWidget *parent, cchar *where, cchar *message, ...);       //  show message, return choice  
+void poptext_screen(cchar *text, int px, int py, float s1, float s2);            //  show popup text at screen posn
+void poptext_mouse(cchar *text, int dx, int dy, float s1, float s2);             //   " " " at mouse posn + offset
+void poptext_window(GtkWindow *, cchar *tx, int x, int y, float s1, float s2);   //   " " " at window posn + offset
+void poptext_widget(GtkWidget *, cchar *tx, int x, int y, float s1, float s2);   //   " " " at widget posn + offset
+int  poptext_killnow();                                                          //  kill current popup window
+int  popup_image(cchar *imagefile, GtkWindow *parent, int Fnewin, int size);     //  popup window with image
 
 //  file chooser dialogs for one file or multiple files
 
